#! -*- coding: utf-8 -*-
"""Admintool to insert target values into beneficiary table

Attributes:

    WRONG_FORMAT -- Error Message

Example:

    python set_targets.py $development.ini $target_values.txt

"""

import argparse
import csv
import decimal

from pyramid.paster import get_appsettings as load_settings
from sqlalchemy import engine_from_config
from sqlalchemy.sql import text


WRONG_FORMAT = """Wrong CSV-Format. Error in line: {}
\tExample:$instance_id;$oi_target_number;$ri_target_percent_percent"""


def get_engine_from_configuration(configuration):
    """Retrieve SQL-Alchemy-Engine configuration

    Args:
        configuration -- loaded development.ini

    Returns:
        Engine -- sqlalchemy engine
    """
    config = {"sqlalchemy.url": configuration["sqlalchemy.url"]}
    return engine_from_config(config)


def get_targets_from_csv(csvpath, instance_id):
    """Read target values for oi and ri from csv

    Args:
        csvpath -- location of csv file
        instance_id  -- given instance id

    Returns:
        Tuple -- ($oi, $ri)

    Raises:
        ValueError -- when instance id is not found
    """
    with open(csvpath, 'rb') as csvfile:
        csvreader = csv.reader(csvfile, delimiter=';',
                               quotechar='"',
                               quoting=csv.QUOTE_ALL,
                               skipinitialspace=True)
        for line, row in enumerate(csvreader):
            if not len(row) == 3:
                raise ValueError(WRONG_FORMAT.format(line+1))
            if row[0] == instance_id:
                if instance_id.lower().startswith("ec"):
                    raise Exception("Fuuu EC")
                oi_target_number = row[1]
                ri_target_percent = row[2]
                if not oi_target_number or not ri_target_percent:
                    raise ValueError(
                        "Error in line {}. Value not set correctly".format(
                            line))
                return (oi_target_number, ri_target_percent)
    raise ValueError("Instance id {} not found in csv file".format(
        instance_id))


def prepare_for_db(number):
    """Converts a number to Decimal type and
    multiplies by 10000

    Business Rule:
    We expect percentages in the form of "0,39".
    To display the percentage value on screen we
    divide the value from the db by 100.

    "0,39" results in 3900 as a DB Value

    Args:
        number (string): number

    Returns:
        number as Decimal
    """
    dot_notated = number.replace(",", ".")

    return decimal.Decimal(dot_notated)*10000


def update_beneficiaries(engine, oi_target_number, ri_target_percent,
                         instance_id, dry_run=True):
    """Executes the update

    Args:
        engine (Engine): Database connection
        oi_target_number (string): absolute number of oi targets
        ri_target_percent (string): percantage of ri targets (e.g. "0,39")
        instance_id (string): which beneficiary to update
        dry_run (bool, optional): Defaults to True.
    """
    ri_target_percent = int(prepare_for_db(ri_target_percent))
    if dry_run:
        sql = "UPDATE beneficiarys SET oi_target_number={},"\
              "ri_target_percent={};"
        print sql.format(oi_target_number, ri_target_percent, instance_id)
    else:
        values = {"oi_target_number": oi_target_number,
                  "ri_target_percent": ri_target_percent,
                  "beneficiary_id": instance_id}
        statement = text("UPDATE beneficiarys "
                         "SET oi_target_number=:oi_target_number,"
                         " ri_target_percent=:ri_target_percent ")
        with engine.begin() as connection:
            connection.execute(statement, values)


def main():
    """App Entry
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("config", help="The ini-config of the program.")
    parser.add_argument("csv", help=("path to the csv file. CSV file should "
                                     "be: "
                                     "Vorhabenkennung;ZielwertOI;ZielwertProzentRI"))
    parser.add_argument("--dryrun", help="Don't write something into the db",
                        action='store_const', const=True)
    args = parser.parse_args()

    configuration = load_settings(args.config)
    instance_id = configuration["efa.instance_id"]
    engine = get_engine_from_configuration(configuration)
    oi_target_number, ri_target_percent = get_targets_from_csv(
        args.csv, instance_id)
    update_beneficiaries(engine, oi_target_number, ri_target_percent,
                         instance_id, dry_run=args.dryrun)


if __name__ == "__main__":
    main()
